from sentinelsat.sentinel import SentinelAPI
from sentinelsat.sentinel import read_geojson
from sentinelsat.sentinel import geojson_to_wkt
from datetime import date
from Cryptodome import Random
from Cryptodome.Cipher import AES
import base64
import hashlib
from collections import OrderedDict
import glob
import os
from process_s2 import processImage

bs = AES.block_size
key = 'xdfgfces,kghceilgrhseklh'
key = hashlib.sha256(key.encode()).digest()

def pad(s):
    return s + (bs - len(s) % bs) * chr(bs - len(s) % bs)

def unpad(s):
    return s[:-ord(s[len(s)-1:])]

def encrypt(raw):
    raw = pad(raw)
    iv = Random.new().read(AES.block_size)
    cipher = AES.new(key, AES.MODE_CBC, iv)
    return base64.b64encode(iv + cipher.encrypt(raw.encode())).decode('utf-8')

def decrypt(enc):
    enc = base64.b64decode(enc)
    iv = enc[:AES.block_size]
    cipher = AES.new(key, AES.MODE_CBC, iv)
    return unpad(cipher.decrypt(enc[AES.block_size:])).decode('utf-8')

def printMeta(product_values, items):
    largest = 0
    for el in items:
        if len(el) > largest:
            largest = len(el);
            
    for el in items:
        print(el.ljust(largest) + ': ' + product_values[el])
# main()

usern = decrypt('qKkIABF8lWXs/ohr5YbGzLJkzANbRtaQHePY5O1MxVA=')
passw = decrypt('XWWE92LEzR+HXMVyk1wa4vTnrwFuYOWw/m0euGLMPw8=')
api = SentinelAPI(usern, passw, 'https://scihub.copernicus.eu/dhus')

starttime = date(2020, 6, 5)
endtime = date(2020, 6, 10)
cloudcovermax = 30 # Sentinel-2 only
sentinel1_producttypes = ['GRD']
sentinel2_producttypes = ['S2MSI2A']
sentinel3_producttypes = ['OL_1_EFR___']

aois = glob.glob('aois/*.json')
for aoi in aois:
    print('Processing', aoi)
    footprint = geojson_to_wkt(read_geojson(aoi))
    #footprintname = os.path.basename(aoi)[0:-5]
    footprintname = aoi[0:-5]

    allProducts = OrderedDict()
    # Sentinel-1
    for producttype in sentinel1_producttypes:
        key_values = {
                'platformname': 'Sentinel-1',
                'producttype': producttype,
                'date': (starttime, endtime)}
        products = api.query(footprint, **key_values)
        if len(products) > 0: # keep one while we are developing
            products = dict([list(products.items())[0]])
        #allProducts.update(products)
    # Sentinel-2
    for producttype in sentinel2_producttypes:
        key_values = {
                'platformname': 'Sentinel-2',
                'producttype': producttype,
                'cloudcoverpercentage': (0, cloudcovermax),
                'date': (starttime, endtime)}
        products = api.query(footprint, **key_values)
        #if len(products) > 0: # keep one while we are developing
        #    products = dict([list(products.items())[0]])
        allProducts.update(products)
    # Sentinel-3
    for producttype in sentinel3_producttypes:
        key_values = {
                'platformname': 'Sentinel-3',
                'producttype': producttype,
                'date': (starttime, endtime)}
        products = api.query(footprint, **key_values)
        if len(products) > 0: # keep one while we are developing
            products = dict([list(products.items())[0]])
        #allProducts.update(products)

    print('Query for \'' + footprintname + '\' returned ' + str(len(allProducts)) + ' item(s)')
    print()

    if not os.path.exists(footprintname):
        os.makedirs(footprintname)

    for product_key, product_values in allProducts.items():
        printMeta(product_values, ['uuid','title','platformname','instrumentshortname','producttype','footprint'])
        sentinelfile = product_values['filename'][0:-5]
        zipfile = sentinelfile + '.zip'
        print(zipfile)
        if not os.path.exists(footprintname + '/' + zipfile) and not os.path.exists(footprintname + '/' + product_values['filename']):
            api.download(product_key, footprintname)
        else:
            print('File already downloaded ... skipping')
        processImage(footprintname, sentinelfile)
        print()


## download single scene by known product id
#api.download(<product_id>)

## download all results from the search
#api.download_all(products)
