from zipfile import ZipFile
import xml.etree.ElementTree as xml
from osgeo import gdal
from gdal import gdalconst
from uuid import uuid4
import numpy as np
import os
from math import radians
from main_Inversion2SeaColor_S2 import TwoSeaColor
import timeit

#sentinelfile = 'S2B_MSIL2A_20200527T113319_N0214_R080_T28PBA_20200527T140657'

def processImage(path, sentinelfile):
    band = sentinelfile
    if (band.startswith('S2')):
        band = band.split('_')
        band = band[5] + '_' + band[2]
        bands = []
        bands.append(band + '_B01_60m')
        bands.append(band + '_B02_10m')
        bands.append(band + '_B03_10m')
        bands.append(band + '_B04_10m')
        bands.append(band + '_B05_20m')
        bands.append(band + '_B06_20m')
        bands.append(band + '_B07_20m')
        bands.append(band + '_B08_10m')
        #bands.append(band + '_B10_10m')
        bands.append(band + '_B11_20m')

    print(path)
    print(sentinelfile)
    filename = (path + '/' + sentinelfile + '.zip')
    z = ZipFile(filename)

    manifest = z.open(sentinelfile + '.SAFE/manifest.safe')
    manifest = xml.parse(manifest).getroot()
    tile_metadata = manifest.find("dataObjectSection/dataObject[@ID='S2_Level-2A_Tile1_Metadata']/byteStream/fileLocation")
    tile_metadata = tile_metadata.get('href').lstrip('./')
    tile_metadata = z.open(sentinelfile + '.SAFE/' + tile_metadata)
    tile_metadata = xml.parse(tile_metadata).getroot()
    namespace_metadata = tile_metadata.tag.split('}')[0] + '}'
    utmzone = tile_metadata.find(namespace_metadata + 'Geometric_Info/Tile_Geocoding/HORIZONTAL_CS_NAME').text
    epsg = tile_metadata.find(namespace_metadata + 'Geometric_Info/Tile_Geocoding/HORIZONTAL_CS_CODE').text
    rows10 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Size[@resolution='10']/NROWS").text)
    cols10 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Size[@resolution='10']/NCOLS").text)
    rows20 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Size[@resolution='20']/NROWS").text)
    cols20 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Size[@resolution='20']/NCOLS").text)
    rows60 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Size[@resolution='60']/NROWS").text)
    cols60 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Size[@resolution='60']/NCOLS").text)

    ulx10 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='10']/ULX").text)
    uly10 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='10']/ULY").text)
    xdim10 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='10']/XDIM").text)
    ydim10 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='10']/YDIM").text)

    ulx20 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='20']/ULX").text)
    uly20 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='20']/ULY").text)
    xdim20 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='20']/XDIM").text)
    ydim20 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='20']/YDIM").text)

    ulx60 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='60']/ULX").text)
    uly60 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='60']/ULY").text)
    xdim60 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='60']/XDIM").text)
    ydim60 = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Geocoding/Geoposition[@resolution='60']/YDIM").text)

    solar_zenith_colstep = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Angles/Sun_Angles_Grid/Zenith/COL_STEP").text)
    solar_zenith_rowstep = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Angles/Sun_Angles_Grid/Zenith/ROW_STEP").text)
    solar_zenith_angles = tile_metadata.findall(namespace_metadata + "Geometric_Info/Tile_Angles/Sun_Angles_Grid/Zenith/Values_List/VALUES")
    solar_zenith_angles = [[float(f) for f in z.text.split(' ')] for z in solar_zenith_angles]

    solar_azimuth_colstep = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Angles/Sun_Angles_Grid/Azimuth/COL_STEP").text)
    solar_azimuth_rowstep = int(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Angles/Sun_Angles_Grid/Azimuth/ROW_STEP").text)
    solar_azimuth_angles = tile_metadata.findall(namespace_metadata + "Geometric_Info/Tile_Angles/Sun_Angles_Grid/Azimuth/Values_List/VALUES")
    solar_azimuth_angles = [[float(f) for f in a.text.split(' ')] for a in solar_azimuth_angles]

    # for now use the average values instead
    solar_zenith_angle = float(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Angles/Mean_Sun_Angle/ZENITH_ANGLE").text)
    solar_azimuth_angle = float(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Angles/Mean_Sun_Angle/AZIMUTH_ANGLE").text)
    viewing_zenith_angle = float(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Angles/Mean_Viewing_Incidence_Angle_List/Mean_Viewing_Incidence_Angle[@bandId='0']/ZENITH_ANGLE").text)
    viewing_azimuth_angle = float(tile_metadata.find(namespace_metadata + "Geometric_Info/Tile_Angles/Mean_Viewing_Incidence_Angle_List/Mean_Viewing_Incidence_Angle[@bandId='0']/AZIMUTH_ANGLE").text)

    msil = z.open(sentinelfile + '.SAFE/MTD_MSIL2A.xml')
    msil = xml.parse(msil).getroot()
    namespace_msil = msil.tag.split('}')[0] + '}'
    scalefactor = float(msil.find(namespace_msil + "General_Info/Product_Image_Characteristics/QUANTIFICATION_VALUES_LIST/BOA_QUANTIFICATION_VALUE").text)
    filenames = msil.findall(namespace_msil + 'General_Info/Product_Info/Product_Organisation/Granule_List/Granule/IMAGE_FILE')

    # Read in memory
    rasters = []
    mmap_names = []
    geotransform = None
    projection = None
    for band in bands:
        for name in filenames:
            name = name.text + '.jp2'
            if band in name:
                print(name)
                f = z.open(sentinelfile + '.SAFE/' + name)
                mmap_name = "/vsimem/"+name
                gdal.FileFromMemBuffer(mmap_name, f.read())
                dataset=gdal.Open(mmap_name)
                if not band.endswith('_10m'):
                    print("dataset: ", dataset.RasterXSize, "x", dataset.RasterYSize)
                    rasterband = dataset.GetRasterBand(1)
                    dataset = gdal.Warp('', mmap_name, format='MEM', resampleAlg = gdalconst.GRA_Bilinear, outputType=rasterband.DataType, xRes=10, yRes=10)
                    rasterband = None
                    print("warped to: ", dataset.RasterXSize, "x", dataset.RasterYSize)
                else:
                    print("dataset: ", dataset.RasterXSize, "x", dataset.RasterYSize)
                    if geotransform == None:
                        geotransform = dataset.GetGeoTransform()
                        projection = dataset.GetProjection()
                rasterband = dataset.GetRasterBand(1)
                rasterdata = rasterband.ReadAsArray()
                rasters.append(rasterdata)
                mmap_names.append(mmap_name)
    # Process
    sza=radians(solar_zenith_angle) # Solar viweing angle: read from the image
    vza=radians(viewing_zenith_angle)  # Viewing zenith angle: read from the iamge
    saa=radians(solar_azimuth_angle)  #Solar azimuth : read from the image
    vaa=radians(viewing_azimuth_angle)  #Viewing azimuth : read from the iamge
    tsc = TwoSeaColor(sza, vza, saa, vaa)
    nr_bands = 3 * 8 + 5
    out_data = np.empty((nr_bands,cols10,rows10)) # 4 output bands
    for y in range(0,rows10):
        start = timeit.default_timer()
        water = 0
        for x in range(0,cols10):
            values = [raster[x,y] / scalefactor for raster in rasters] # the data at x,y for B01..B08, and B11 (B11 only included for water-mask threshold)
            # water/sea thresholds:
            if values[2] < 0.325 and values[6] < 0.267 and values[8] < 0.085: # B03, B07, B11
                values.pop() # remove B11
                water = water + 1
                # call 2seacolor
                Rrs, R, Kd, IOP = tsc.process(values)
                # store result
                for i in range(0,8):
                    out_data[i,x,y] = Rrs[i]
                    out_data[i + 8,x,y] = R[i]
                    out_data[i + 16,x,y] = Kd[i]
                for i in range(0,5):
                    out_data[i + 24,x,y] = IOP[i]
            else:
                for i in range(0, nr_bands):
                    out_data[i,x,y] = -1
        stop = timeit.default_timer()
        execution_time = stop - start
        print(y, 'of', rows10, 'completed in', str(execution_time), 'seconds; ', water, 'water pixels; est. remaining: ', (rows10 - y) * execution_time, 'seconds')

    print(rows10, 'of', rows10, 'completed')
    driver = gdal.GetDriverByName('GTiff')
    dataset_out = driver.Create(path + '/' + band + '_2seacolor.tif', cols10, rows10, nr_bands, gdal.GDT_Float64)
    dataset_out.SetGeoTransform(geotransform)
    dataset_out.SetProjection(projection)
    for i in range(0,nr_bands):
        dataset_out.GetRasterBand(i+1).WriteArray(out_data[i])
    dataset_out.FlushCache()  # Write to disk.
    dataset_out = None
    rasters = None

    # Release memory
    for mmap_name in mmap_names:
        gdal.Unlink(mmap_name)

